<?php
/*
	Typographical Improvements

	Modifies text generally and especially under typographical considerations.


	Plugin Name: Typographical Improvements
	Plugin URI: http://rabich.de/typographical-improvements/
	Description: Das Plugin verbessert die Darstellung des Texts. Insbesondere Anf&uuml;hrungszeichen, Bindestriche, Leerzeichen sowie spezielle Sonderzeichen werden angepasst.
	Version: 0.6c
	Author: Dietmar Rabich
	Author URI: http://rabich.de/


	Copyright (c) 2008 Dietmar Rabich, Duelmen, Germany

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License (version 2) as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/



// ============================================================
// here i am
// ============================================================
define('TYPOGRAPHICALIMPROVEMENTS', true);



// ============================================================
// Localization
// ============================================================

$ti_domain	= 'typographical-improvements';
$ti_is_setup	= false;

// setup
function ti_setup() {
	global $ti_domain, $ti_is_setup;

	if ( $ti_is_setup ) {
		return;
	}

	load_plugin_textdomain($ti_domain, 'wp-content/plugins');

	$ti_is_setup	= true;
}

// output
function ti__($s) {
	global $ti_domain;

	return __($s, $ti_domain);
}

function ti_e($s) {
	global $ti_domain;

	_e($s, $ti_domain);
}



// ============================================================
// Global variables
// ============================================================

// language
$ti_lang = '';



// ============================================================
// Utility functions
// ============================================================

// short locale
function _get_short_locale() {
	return preg_replace("/[_\-].*$/", '', get_locale());
}



// prepare for regexp
function _torx($s) {
	return str_replace('#', '\#', $s);
}



// ============================================================
// Improvement functions
// ============================================================

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// alternative quotes
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function _alt_quotes($text) {

	global $ti_quotes, $ti_lang;

	// first the secondary quotes (special way because of the apostroph)
	$opt_secquoteopen = get_option("ti_secquoteopen");
	if ( $opt_secquoteopen !== false && !empty($opt_secquoteopen) ) {
		if ( $ti_quotes[$ti_lang]['so'] == '&#8217;' ) {
			$text	= preg_replace("/ (\s|" . _torx($ti_quotes[$ti_lang]['po']) . "|\A) &\#8217; /x",
						"$1" . $opt_secquoteopen,
						$text);
		}
		else
			$text	= str_replace($ti_quotes[$ti_lang]['so'], $opt_secquoteopen, $text);
	}

	$opt_secquoteclose = get_option("ti_secquoteclose");
	if ( $opt_secquoteclose !== false && !empty($opt_secquoteclose) ) {
		if ( $ti_quotes[$ti_lang]['sc'] == '&#8217;' ) {
			$text = preg_replace("/ &\#8217; (?=[\s\.!\?\:,;]|" . _torx($ti_quotes[$ti_lang]['pc']) . "|\Z) /x", $opt_secquoteclose, $text);
		}
		else
			$text	= str_replace($ti_quotes[$ti_lang]['sc'], $opt_secquoteclose, $text);
	}

	// then the primary quotes
	$opt_primquoteopen = get_option("ti_primquoteopen");
	if ( $opt_primquoteopen !== false && !empty($opt_primquoteopen) ) {
		$text	= str_replace($ti_quotes[$ti_lang]['po'], $opt_primquoteopen, $text);
	}

	$opt_primquoteclose = get_option("ti_primquoteclose");
	if ( $opt_primquoteclose !== false && !empty($opt_primquoteclose) ) {
		$text	= str_replace($ti_quotes[$ti_lang]['pc'], $opt_primquoteclose, $text);
	}


	return $text;
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// text optimization (international with german touch)
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function _improve_text_international($text) {

	global $ti_repls;

	$opt_guillemets = get_option("ti_guillemets");
	if ( $opt_guillemets === false )
		$opt_guillemets	= 1;

	$opt_speed = get_option("ti_speed");
	if ( $opt_speed === false )
		$opt_speed	= 0;

	$opt_specialchar1 = get_option("ti_specialchar1");
	if ( $opt_specialchar1 === false )
		$opt_specialchar1	= 1;

	$opt_specialchar2 = get_option("ti_specialchar2");
	if ( $opt_specialchar2 === false )
		$opt_specialchar2	= 1;

	$opt_specialchar3 = get_option("ti_specialchar3");
	if ( $opt_specialchar3 === false )
		$opt_specialchar3	= 0;


	// special characters
	if ( $opt_specialchar1 )
		$text = preg_replace($ti_repls['spec1']['s'], $ti_repls['spec1']['r'], $text);

	if ( $opt_specialchar2 )
		$text = preg_replace($ti_repls['spec2']['s'], $ti_repls['spec2']['r'], $text);

	if ( $opt_specialchar3 )
		$text = preg_replace($ti_repls['spec3']['s'], $ti_repls['spec3']['r'], $text);


	// quotation marks
	if ( strpos($text, '"') !== false || strpos($text, "'") !== false ) {

		// more common but not real quotation marks
		$text = preg_replace($ti_repls['notrealquot']['s'], $ti_repls['notrealquot']['r'], $text);


		// quotation marks
		if ( $opt_speed ) {
			$text = preg_replace($ti_repls['quot_sp']['s'], $ti_repls['quot_sp']['r'], $text);
		} else {
			$text = preg_replace($ti_repls['quot']['s'], $ti_repls['quot']['r'], $text);
		}


		// possibly forgotten primes
		$text = preg_replace($ti_repls['quot_fg1']['s'], $ti_repls['quot_fg1']['r'], $text);


		// possibly forgotten quotation marks and apostrophs
		if ( $opt_speed ) {
			$text = preg_replace($ti_repls['quot_fg2sp']['s'], $ti_repls['quot_fg2sp']['r'], $text);
		} else {
			$text = preg_replace($ti_repls['quot_fg2']['s'], $ti_repls['quot_fg2']['r'], $text);
			$text = str_replace($ti_repls['quot_fg3']['s'], $ti_repls['quot_fg3']['r'], $text);
			$text = preg_replace($ti_repls['quot_fg4']['s'], $ti_repls['quot_fg4']['r'], $text);
			$text = str_replace($ti_repls['quot_fg5']['s'], $ti_repls['quot_fg5']['r'], $text);
		}

	}


	// guillemets
	if ( ($opt_guillemets == 1) && ($poslt = strpos($text, '&lt;')) !== false && ($posgt = strpos($text, '&gt;')) !== false ) {

		$text = ( $opt_speed ) ?
				str_replace($ti_repls['guill_sp']['s'], $ti_repls['guill_sp']['r'], $text) :
				( $poslt < $posgt ) ?
					preg_replace($ti_repls['guill_lg']['s'], $ti_repls['guill_lg']['r'], $text) :	// <<...>>, <...>
					preg_replace($ti_repls['guill_gl']['s'], $ti_repls['guill_gl']['r'], $text);	// >>...<<, >...<

	}

	// omitted words
	$text = str_replace($ti_repls['omitted']['s'], $ti_repls['omitted']['r'], $text);


	// hyphens, dashes
	$text = preg_replace($ti_repls['dashes']['s'], $ti_repls['dashes']['r'], $text);


	// ratios
	$text = preg_replace($ti_repls['ratios']['s'], $ti_repls['ratios']['r'], $text);


	// date and time
	$text = preg_replace($ti_repls['datetime']['s'], $ti_repls['datetime']['r'], $text);


	// currencies
	$text = preg_replace($ti_repls['currencies']['s'], $ti_repls['currencies']['r'], $text);


	// dimensions
	$text = preg_replace($ti_repls['dimensions']['s'], $ti_repls['dimensions']['r'], $text);


	// per cent, degree etc.
	$text = preg_replace($ti_repls['percent']['s'], $ti_repls['percent']['r'], $text);


	// numbers
	$text = preg_replace($ti_repls['numbers']['s'], $ti_repls['numbers']['r'], $text);


	// spaces
	$text = preg_replace($ti_repls['spaces']['s'], $ti_repls['spaces']['r'], $text);


	// alternative quotes
	$text = _alt_quotes($text);

	return $text;
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// text improvements
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function _improve_text($text) {

	global $ti_lang, $ti_repls, $ti_quotes;
	$cs	= get_settings('blog_charset');


	// protects existing entities
	$textarr	= preg_split("/(&(?:#(?:[0-9]+|[xX][0-9A-Fa-f]+)|[A-Za-z0-9]+);)/Us", $text, -1, PREG_SPLIT_DELIM_CAPTURE);
	$text	= '';

	for ($i = 0; $i < count($textarr); $i++) {

		if ( isset($textarr[$i]{0}) && !preg_match("/^(?:&(?:#(?:[0-9]+|[xX][0-9A-Fa-f]+)|[A-Za-z0-9]+);)$/Us", $textarr[$i]) ) {
			$textarr[$i] = htmlentities($textarr[$i], ENT_NOQUOTES, $cs);
		}

		$text .= $textarr[$i];
	}


	// special replacements for everyone
	$text = str_replace($ti_repls['special']['s'], $ti_repls['special']['r'], $text);


	// language dependent replacements
	if ( array_key_exists($ti_lang, $ti_quotes) ) {
		$text	= _improve_text_international($text);
	}


	return $text;
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// replaces many special characters
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function ti_replace($text) {

	// output string
	$output = '';

	// capture tags and everything inside them
	$textarr = preg_split("/(<[^<]*?>|\[[^\[]*?\]|\[{2}[^\[]*?\]{2})/s", $text, -1, PREG_SPLIT_DELIM_CAPTURE);

	$leapfrog = '';	// loop stuff for preformatted text

	for ( $i = 0; $i < count($textarr); $i++ ) {

		$curl = $textarr[$i];

		// Text, not tag
		if ( isset($curl{0}) && '<' != $curl{0} && '[' != $curl{0} && $leapfrog == '' ) {
			$curl	= _improve_text($curl);
		}

		// preformatted
		elseif ( preg_match("/^<(code|pre|kbd|samp|style|script)/", $curl, $matchingtag) ) {
			$leapfrog = $matchingtag[1];
		}

		// preformatted end
		elseif ( preg_match("/^<\/$leapfrog/", $curl) ) {
			$leapfrog = '';
		}

		// forgotten ampersand
		$curl = preg_replace('/&([^#])(?![a-zA-Z1-4]{1,8};)/', '&#038;$1', $curl);

		// append to output
		$output .= $curl;
	}

	return $output;
}



// ============================================================
// Initialization
// ============================================================

function ti_init() {
	global $ti_lang, $ti_repls, $ti_quotes;

	// language
	$ti_lang = _get_short_locale();

	// language dependent replacements
	if ( in_array($ti_lang, array('af', 'bg', 'cs', 'de', 'hu', 'is', 'lt', 'nl', 'pl', 'ro', 'sk', 'sl', 'sr')) ) {
		$ti_lang = 'de';
	} elseif ( in_array($ti_lang, array('en', 'eo', 'ga')) ) {
		$ti_lang = 'en';
	} elseif ( in_array($ti_lang, array('fi', 'sv')) ) {
		$ti_lang = 'sv';
	} elseif ( in_array($ti_lang, array('ar', 'be', 'el', 'es', 'et', 'it', 'lv', 'no', 'pt', 'ro', 'sq', 'tr', 'uk')) ) {
		$ti_lang = 'es';
	} elseif ( in_array($ti_lang, array('fr')) ) {
		$ti_lang = 'fr';
	} elseif ( in_array($ti_lang, array('dk', 'hr')) ) {
		$ti_lang = 'dk';
	} else {
		$ti_lang = '';
	}
	
	// national quotes (primary open and close, secondary open and close)
	$ti_quotes = array(
	'de'	=> array('po' => '&#8222;', 'pc' => '&#8220;', 'so' => '&#8218;', 'sc' => '&#8216;'),	// german
	'en'	=> array('po' => '&#8220;', 'pc' => '&#8221;', 'so' => '&#8216;', 'sc' => '&#8217;'),	// english
	'sv'	=> array('po' => '&#8221;', 'pc' => '&#8221;', 'so' => '&#8217;', 'sc' => '&#8217;'),	// swedish
	'es'	=> array('po' => '&#171;', 'pc' => '&#187;', 'so' => '&#8249;', 'sc' => '&#8250;'),	// spanish
	'dk'	=> array('po' => '&#187;', 'pc' => '&#171;', 'so' => '&#8250;', 'sc' => '&#8249;'),	// danish
	'fr'	=> array('po' => '&#171;&#160;', 'pc' => '&#160;&#187;', 'so' => '&#8249;&#160;', 'sc' => '&#160;&#8250;')	// french
	);
	
	// the replacements ...
	$ti_repls = array(
	'special'	=> array(
			's' => array('----',
					'---',
					' -- ',
					'--',
					'xn&#8211;',
					'&deg;',
					'&sect;',
					'&euro;',
					'&amp;lt;',
					'&amp;gt;'),
			'r' => array('&#8213;',
					'&#8212;',
					' &#8212; ',
					'&#8211;',
					'xn--',
					"&#176;",
					"&#167;",
					"&#8364;",
					"&lt;",
					"&gt;")
		),
	'spec1'	=> array(
			's' => array("/\s*\(tm\)/i",
					"/\s*\(r\)/i",
					"/(?:(?<=copyright )\(c\)|\(c\)(?=\s\d{4}))/i",
					'/(\d+)\s*x\s*(\d+)/',				// times
					'/(\S+)\s(&(?:amp|\#38);|&)\s(\S+)/',		// ampersand
					'/(\d+)\s*\/\s*(\d+)/',				// fraction
					'/(?<=\d)\s(m)?l(?=\s|\Z|[\.!\?\:,;])/',	// liter
					'/\+\/\-/',					// plus-minus
					'/&\#176;C(?=\Z|[\s\.!\?\:,;])/',		// degree Celsius
					'/&\#176;F(?=\Z|[\s\.!\?\:,;])/',		// degree Fahrenheit
					'/(?<=\d&\#160;)K(?=\Z|[\s\.!\?\:,;])/',	// Kelvin
					"/ (\s|\A) 1(?:\/|&\#8260;)4 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",	// fraction
					"/ (\s|\A) 1(?:\/|&\#8260;)2 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 3(?:\/|&\#8260;)4 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 1(?:\/|&\#8260;)3 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 2(?:\/|&\#8260;)3 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 1(?:\/|&\#8260;)5 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 2(?:\/|&\#8260;)5 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 3(?:\/|&\#8260;)5 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 4(?:\/|&\#8260;)5 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 1(?:\/|&\#8260;)6 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 5(?:\/|&\#8260;)6 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 1(?:\/|&\#8260;)8 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 3(?:\/|&\#8260;)8 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 5(?:\/|&\#8260;)8 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) 7(?:\/|&\#8260;)8 (?=[\s\-\.!\?,;\:]|&\#(?:8208|8210|8211|8212|8213);|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:I([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",	// roman numbers
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:II([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:III([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:IV([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:V([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:VI([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:VII([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:VIII([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:IX([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:X([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:XI([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:XII([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:L([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:C([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:D([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:M([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:i([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:ii([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:iii([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:iv([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:v([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:vi([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:vii([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:viii([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:ix([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:x([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:xi([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:xii([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:l([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:c([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:d([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x",
					"/ (\s|\A) ((\()? (\[)? (\{)?) (?:m([\.\:]?)) ((?(3) \) | (?(4) \] | (?(5) \} | [\)]?)))) (\s|\Z) /x"),
			'r' => array('&#8482;',
					'&#174;',
					'&#169;',
					"$1&#160;&#215;&#160;$2",
					"$1&#160;&#38;&#160;$3",
					"$1&#8260;$2",
					"&#160;$1&#8467;",
					"&#177;",
					"&#8451;",
					"&#8457;",
					"&#8490;",
					"$1&#188;",
					"$1&#189;",
					"$1&#190;",
					"$1&#8531;",
					"$1&#8532;",
					"$1&#8533;",
					"$1&#8534;",
					"$1&#8535;",
					"$1&#8536;",
					"$1&#8537;",
					"$1&#8538;",
					"$1&#8539;",
					"$1&#8540;",
					"$1&#8541;",
					"$1&#8542;",
					"$1$2&#8544;$6$7$8",
					"$1$2&#8545;$6$7$8",
					"$1$2&#8546;$6$7$8",
					"$1$2&#8547;$6$7$8",
					"$1$2&#8548;$6$7$8",
					"$1$2&#8549;$6$7$8",
					"$1$2&#8550;$6$7$8",
					"$1$2&#8551;$6$7$8",
					"$1$2&#8552;$6$7$8",
					"$1$2&#8553;$6$7$8",
					"$1$2&#8554;$6$7$8",
					"$1$2&#8555;$6$7$8",
					"$1$2&#8556;$6$7$8",
					"$1$2&#8557;$6$7$8",
					"$1$2&#8558;$6$7$8",
					"$1$2&#8559;$6$7$8",
					"$1$2&#8560;$6$7$8",
					"$1$2&#8561;$6$7$8",
					"$1$2&#8562;$6$7$8",
					"$1$2&#8563;$6$7$8",
					"$1$2&#8564;$6$7$8",
					"$1$2&#8565;$6$7$8",
					"$1$2&#8566;$6$7$8",
					"$1$2&#8567;$6$7$8",
					"$1$2&#8568;$6$7$8",
					"$1$2&#8569;$6$7$8",
					"$1$2&#8570;$6$7$8",
					"$1$2&#8571;$6$7$8",
					"$1$2&#8572;$6$7$8",
					"$1$2&#8573;$6$7$8",
					"$1$2&#8574;$6$7$8",
					"$1$2&#8575;$6$7$8")
	),
	'spec2'	=> array(
			's' => array("/ (\s|&\#160;|\A) &lt;- (?=\s|&\#160;|\Z) /x",
					"/ (\s|&\#160;|\A) -&gt; (?=\s|&\#160;|\Z) /x",
					"/ (\s|&\#160;|\A) c\/o (?=\s|&\#160;|\Z) /x",
					"/ (\s|&\#160;|\d|\A) (?:%o|o\/oo|0(?:\/|&\#8260;)00) (?=[\s\-\.!\?,;\:]|\Z) /x",
					"/ (\s|&\#160;|\d|\A) (?:%oo|o\/ooo|0(?:\/|&\#8260;)000) (?=[\s\-\.!\?,;\:]|\Z) /x",
					"/ \s (?=&\#(?:8240|8241);(?:[\s\-\.!\?,;\:]|\Z)) /x",
					"/ (\s|&\#160;|\A) &lt;= (?=\s|&\#160;|\Z) /x",
					"/ (\s|&\#160;|\A) &gt;= (?=\s|&\#160;|\Z) /x",
					"/ (\s|&\#160;|\A) (?:!=|&lt;&gt;) (?=\s|&\#160;|\Z) /x"),
			'r' => array("$1&#8592;",
					"$1&#8594;",
					"$1&#8453;",
					"$1&#8240;",
					"$1&#8241;",
					"&#160;",
					"$1&#8804;",
					"$1&#8805;",
					"$1&#8800;")
	),
	'spec3'	=> array(
			's' => array("/ (\s|\A) (?:\:\-\(|\)\-\:) (?=\s|\Z) /x",
					"/ (\s|\A) (?:\:\-\)|\(\-\:) (?=\s|\Z) /x"),
			'r' => array("$1&#9785;",
					"$1&#9786;")
	),
	'notrealquot'	=> array(
			's' => array("/ (?<!\s) ' (?=[mns](?:\Z|[\s\.!\?\:,;])) /x",		// mostly awful apostrophe (like CD's)
					'/ (\s|\A)
							(\d+)&\#176; \s*
							(\d+)\' \s*
							(\d+)"
						(?=\Z|[\s\.!\?\:,;]) /x',			// degree (like 54(o) 23' 17")
					'/ (\s|\A)
							(\d+)\' \s*
							(\d+)"
						(?=\Z|[\s\.!\?\:,;]) /x',			// Feets and inches (like 4' 5")
					'/ (\s|\A) (\d+)" (?=-\S) /x'),				// Inches (like 17"-screen)
			'r' => array('&#8217;',
					'$1$2&#176;&#160;$3&#8242;&#160;$4&#8243;',
					'$1$2&#8242;&#160;$3&#8243;',
					'$1$2&#8243;')
		),
	'quot_sp'	=> array(
			's' => array('/ (\s|\A) ("|`{2}) /x',					// left double quotation mark
					"/ (\"|'{2}) (?=[\s\.!\?\:,;]|\Z) /x",			// right double quotation mark
					"/ (\s|" . _torx($ti_quotes[$ti_lang]['po']) . "|\A) ' /x",	// left single quotation marks
					"/ ' (?=[\s\.!\?\:,;]|" . _torx($ti_quotes[$ti_lang]['pc']) . "|\Z) /x"),	// right single quotation marks
			'r' => array("$1" . $ti_quotes[$ti_lang]['po'],
					$ti_quotes[$ti_lang]['pc'],
					"$1" . $ti_quotes[$ti_lang]['so'],
					$ti_quotes[$ti_lang]['sc'])
		),
	'quot'		=> array(
			's' => array('/ (\s|\A) `{2} (?!`) /x',				// left double quotation mark
					"/ (?<!') '{2} (?=[\s\.!\?\:,;]|\Z) /x",	// right double quotation mark
					'/ (\s|\A) " (?!["\s]) /x',				// left double quotation marks
					'/ (?<!") " (?=[\s\.!\?\:,;]|\Z) /x',		// right double quotation marks
					"/ (\s|" . _torx($ti_quotes[$ti_lang]['po']) . "|\A) ' (?!') /x",	// left single quotation marks
					"/ (?<!') ' (?=[\s\.!\?\:,;]|" . _torx($ti_quotes[$ti_lang]['pc']) . "|\Z) /x"),	// right single quotation marks
			'r' => array("$1" . $ti_quotes[$ti_lang]['po'],
					$ti_quotes[$ti_lang]['pc'],
					"$1" . $ti_quotes[$ti_lang]['po'],
					$ti_quotes[$ti_lang]['pc'],
					"$1" . $ti_quotes[$ti_lang]['so'],
					$ti_quotes[$ti_lang]['sc'])
		),
	'quot_fg1'	=> array(
			's' => array('/(\d+)"/',					// possibly forgotten double primes
					"/(\d+)'/"),					// possibly forgotten primes
			'r' => array('$1&#8243;',
					'$1&#8242;')
		),
	'quot_fg2sp'	=> array(
			's' => array("/ (?<=\w) ' (?=\w) /x"),				// forgotten apostrophs
			'r' => array('&#8217;')
		),
	'quot_fg2'	=> array(
			's' => array('/ (\s|\A|"|&\#(?:171|187|8220|8221|8222|8249|8250);)
								\'
						(?=\S)/x',				// possibly forgotten left single quotation mark
					'/ (?<=\S)
								\'
						(?=\s|\Z|"|&\#(?:171|187|8220|8221|8222|8249|8250);) /x',	// forgotten right single quotation mark
					'/ (\s|\A) " (?=\S) /x'),			// forgotten left double quotation mark
			'r' => array('$1' . $ti_quotes[$ti_lang]['so'],
					$ti_quotes[$ti_lang]['sc'],
					'$1' . $ti_quotes[$ti_lang]['po'])
		),
	'quot_fg3'	=> array(
			's' => array('"'),						// forgotten right double quotation mark (last chance)
			'r' => array($ti_quotes[$ti_lang]['pc'])
		),
	'quot_fg4'	=> array(
			's' => array("/ (?<=\w) ' (?=\w) /x"),				// forgotten apostrophs
			'r' => array('&#8217;')
		),
	'quot_fg5'	=> array(
			's' => array("'"),						// forgotten right single quotation mark (last chance)
			'r' => array($ti_quotes[$ti_lang]['sc'])
		),
	'guill_sp'	=> array(
			's' => array('&lt;&lt;', '&gt;&gt;', '&lt;', '&gt;'),
			'r' => array('&#171;', '&#187;', '&#8249;', '&#8250;')
		),
	'guill_lg'	=> array(
			's' => array('/ (\s|\A) (?:&lt;){2} (?=(?:&lt;)?\S) /x',		// left pointing guillemet
					'/ (\S(?:&gt;)?) (?:&gt;){2} (?=[\s\.!\?\:,;]|\Z) /x',	// right pointing guillemet
					'/ (\s|\A|&\#171;) (?:&lt;) (?=\S) /x',			// left pointing single guillemet
					'/ (?<=\S) (?:&gt;) (?=&\#187;|[\s\.!\?\:,;]|\Z) /x'),	// right pointing single guillemet
			'r' => array("$1&#171;",
					"$1&#187;",
					"$1&#8249;",
					"&#8250;")
		),
	'guill_gl'	=> array(
			's' => array('/ (\s|\A) (?:&gt;){2} (?=(?:&gt;)?\S) /x',		// right pointing guillemet
					'/ (\S(?:&lt;)?) (?:&lt;){2} (?=[\s\.!\?\:,;]|\Z) /x',	// left pointing guillemet
					'/ (\s|\A|&\#187;) (?:&gt;) (?=\S) /x',			// right pointing single guillemet
					'/ (?<=\S) (?:&lt;) (?=&\#171;|[\s\.!\?\:,;]|\Z) /x'),	// left pointing single guillemet
			'r' => array("$1&#187;",
					"$1&#171;",
					"$1&#8250;",
					"&#8249;")
		),
	'omitted'	=> array(
			's' => array(' ...',				// three dots (like more and more ...)
					'...'),
			'r' => array('&#160;&#8230;',
					'&#8230;')
		),
	'dashes'	=> array(
			's' => array("/ (?<=\s)
						- \s ([^\.!\?;]+) \s -
					(?=\Z|[\s\.!\?\:,;]) /x",	// dash (like and - I think so - this is it)
					"/ \s -
					(?=\Z|[\s,]) /x",		// dash
					"/(?<=\w)-(?=\W|\Z)/",		// at the end of a word (like in- or output)
					"/(?<=\s)-(?=\s|\Z)/",		// (like here - there)
					'/ (?<=\d)
						\s* (?:-|&\#8209;|&\#8211;) \s*
					(?=\d)/x',			// (like 5-6)
					"/ (?<=\d\.)
						\s* (?:-|&\#8209;|&\#8211;) \s*
					(?=\d\.) /x",			// (like 5.-6.)
					"/\s-(?!&\#160;)/"),		// possibly forgotten dashes
			'r' => array('&#8211;&#160;$1&#160;&#8211;',
					'&#160;&#8211;',
					'&#8209;',
					'&#8211;',
					'&#8211;',
					'&#8211;',
					'&#160;&#8211;')
		),
	'ratios'	=> array(
			's' => array("/ (?<=\s|\A)
						(\d+)
						\s* : \s*
						( \d{1,3} (?:[\.\s]\d{3})* (?:,\d+)? | \d+ (?:,\d+)? )
					(?=\Z|[\-\s\.!\?\:,;]) /x"),	// (like 1 : 10000)
			'r' => array('$1&#160;:&#160;$2')
		),

	'datetime'	=> array(
			's' => array("/ (\d{1,2})
						(?:&\#160;|\s)*
						[\:\.]
						(?:&\#160;|\s)*
						(\d{2})
						(?:&\#160;|\s)*
						[\:\.]
						(?:&\#160;|\s)*
						(\d{2})
					\s+(?=Uhr|am|pm) /x",		// non-breaking space (like 5:30:05 Uhr)
					"/ (\d{1,2})
						(?:&\#160;|\s)*
						[\:\.]
						(?:&\#160;|\s)*
						(\d{2})
					\s+(?=Uhr|am|pm) /x",		// non-breaking space (like 5:30 Uhr)
					"/ \b
						([0-3]*[0-9]\.)
						\s*
						([0-1]*[0-9]\.)
						\s*
						(\d{4}|&\#8217;\d{2})
					\b /x",				// no spaces (like 17.9.2006)
					"/ \b
						(\d{4}\.)
						(?:-|&\#8211;)
						([0-1][0-9]\.)
						(?:-|&\#8211;)
						([0-3][0-9])
					\b /x"),			// non-breaking hypens (like 2006-09-17)
			'r' => array("$1:$2:$3&#160;",
					"$1:$2&#160;",
					"$1$2$3",
					"$1&#8209;$2&#8209;$3")
		),

	'currencies'	=> array(
			's' => array("/ (?<=\d)
						\s*
						(&\#(?:36|162|163|165|8364);|\\$| (?: (?:US|AU) (?:[\s\-]|&\#8209;)? (?:&\#36;|\\$) ))
					(?=\Z|[\s\.!\?\:,;]) /x",	// non-breaking space (like 5 US$)
					"/ (\s|\A)
						(&\#(?:36|162|163|165|8364);|\\$| (?:(?:US|AU) (?:[\s\-]|&\#8209;)? (?:&\#36;|\\$) ))
						\s*
					(?<=\d) /x"),			// non-breaking space (like US$ 5)
			'r' => array("&#160;$1",
					"$1$2&#160;")
		),
	'dimensions'	=> array(
			's' => array("/ (?<=\d)
						\s+
						([mck]?m|[mk]?g|t|K|&\#(?:8451|8457|8467|8486|8490);|&\#176;[CF]|m?s|h|m?A|m?Ah|cd|mol|m?Pa)
					(?=\Z|[\s\.!\?\:,;]) /x"),	// non-breaking space (like 5 km)
			'r' => array("&#160;$1")
		),
	'percent'	=> array(
			's' => array("/ (?<=\d)
						\s*(%)
					(?=\s|\Z|[\.!\?\:,;]) /x",	// non-breaking space (like 20 %)
					"/ (?<=\d)
						\s(&\#176;)
					(?=\s|\Z|[\.!\?\:,;]) /x"),	// no spaces (like 20(o))
			'r' => array("&#160;$1",
					"$1")
		),
	'numbers'	=> array(
			's' => array("/ (?<=\D|\A)
						(\d{1,3}) \s
					(?=\d{3}(?:\s|\D|\Z)) /x"),		// non-breaking-spaces
			'r' => array("$1$2&#160;$3$4")
		),
	'spaces'	=> array(
			's' => array("/(?<=\(|\/)\s/",			// no space
					"/\s(?=\)|\/)/",		// no space
					"/ ((?:\b(?:Abb|Abs|Anm|Bd|Bl|Nr|S|Sp|Taf|\.)|(?<=\s|\A)(?:&\#167;){1,2}))
					\s
					(?=\d+(?:\D|\Z)|[IVXLCDM](?:\s|\Z)) /x",	// non-breaking space
					"/ ((?:\W|\s|\A)\w\.)
					\s
					(?=\w\.(?:\W|\s|\Z)) /ix"),		// non-breaking space
			'r' => array("",
					"",
					"$1&#160;",
					"$1&#160;")
		),
	);
}



// ============================================================
// Option panel
// ============================================================

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// set option
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function ti_set_option($op, $std, $oname) {
	$opt	= isset($_POST[$op]) ? $_POST[$op] : $std;
	if ( get_option($op) === false )
		add_option($op, $opt, $oname);
	else
		update_option($op, $opt);
	return $opt;
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// get option
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function ti_get_option($op, $def) {
	$opt	= get_option($op);
	if ( $opt === false )
		$opt	= $def;
	return $opt;
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// not so much options
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function ti_options() {
	global $ti_quotes, $ti_lang;

	ti_setup();
?>
	<div class="wrap">
		<h2>Typographical Improvements</h2>

		<p><?php ti_e("Unter den <em>allgemeinen Einstellungen</em> sind die erkannte Sprache und die zugeordneten Anf&uuml;hrungszeichen zu sehen. Die <em>Optionen</em> erlauben die Konfiguration der angewendeten Umwandlungen. Mit den <em>alternativen Anf&uuml;hrungszeichen</em> lassen sich andere Darstellungen oder Zeichen eingeben. Hier ist auch XHTML-Quelltext erlaubt."); ?></p>

		<?php
			if ( isset($_POST['action']) ) {

				$opt_guillemets		= ti_set_option('ti_guillemets', 0, 'Guillemets');
				$opt_speed		= ti_set_option('ti_speed', 0, 'More Speed, less accuracy');
				$opt_specialchar1	= ti_set_option('ti_specialchar1', 0, 'Recognize special characters');
				$opt_specialchar2	= ti_set_option('ti_specialchar2', 0, 'Recognize and convert special characters');
				$opt_specialchar3	= ti_set_option('ti_specialchar3', 0, 'Recognize and convert special characters like faces');
				$opt_primquoteopen	= ti_set_option('ti_primquoteopen', '', 'Primary user defined opening quote');
				$opt_primquoteclose	= ti_set_option('ti_primquoteclose', '', 'Primary user defined closing quote');
				$opt_secquoteopen	= ti_set_option('ti_secquoteopen', '', 'Secondary user defined opening quote');
				$opt_secquoteclose	= ti_set_option('ti_secquoteclose', '', 'Secondary user defined closing quote');

			} else {

				$opt_guillemets		= ti_get_option("ti_guillemets", 1);
				$opt_speed		= ti_get_option("ti_speed", 0);
				$opt_specialchar1	= ti_get_option("ti_specialchar1", 1);
				$opt_specialchar2	= ti_get_option("ti_specialchar2", 1);
				$opt_specialchar3	= ti_get_option("ti_specialchar3", 0);
				$opt_primquoteopen	= ti_get_option("ti_primquoteopen", '');
				$opt_primquoteclose	= ti_get_option("ti_primquoteclose", '');
				$opt_secquoteopen	= ti_get_option("ti_secquoteopen", '');
				$opt_secquoteclose	= ti_get_option("ti_secquoteclose", '');

			}

			$locale = _get_short_locale();

		?>

		<form action="<?php echo $_SERVER[REQUEST_URI] ?>" method="post">
		<input type="hidden" name="action" value="update" />
			<fieldset class="options">
				<legend><?php ti_e("Allgemeine Einstellungen"); ?></legend>
				<table cellspacing="2" cellpadding="5" class="editform">
				<tbody>
					<tr valign="baseline">
						<td scope="row"><?php ti_e("Sprache"); ?>:</td>
						<td><?php echo $locale . " ";
							if ( array_key_exists($ti_lang, $ti_quotes) )
								echo "(" . $ti_quotes[$ti_lang]['po'] . "&#8230;" . $ti_quotes[$ti_lang]['pc'] . ", " . $ti_quotes[$ti_lang]['so'] . "&#8230;" . $ti_quotes[$ti_lang]['sc'] . ")";
							else {
								echo " - " .
									ti__("Diese Sprache wird leider nicht unterst&uuml;tzt.");
							} ?></td>
					</tr>
				</tbody>
				</table>
			</fieldset>
			<fieldset class="options">
				<legend><?php ti_e("Optionen"); ?></legend>
				<table cellspacing="2" cellpadding="5" class="editform">
				<tbody>
					<tr valign="baseline">
						<td><input type="checkbox" id="ti_guillemets" name="ti_guillemets" value="1" <?php if ( $opt_guillemets == 1 ) echo "checked=\"checked\" " ?>/></td>
						<td><label for="ti_guillemets"><?php ti_e("Guillemets erkennen"); ?> (&lt;&#8230;&gt; &rarr; &#8249;&#8230;&#8250;, &lt;&lt;&#8230;&gt;&gt; &rarr; &#171;&#8230;&#187;)</label></td>
					</tr>
					<tr valign="baseline">
						<td><input type="checkbox" id="ti_specialchar1" name="ti_specialchar1" value="1" <?php if ( $opt_specialchar1 == 1 ) echo "checked=\"checked\" " ?>/></td>
						<td><label for="ti_specialchar1"><?php ti_e("Sonderzeichen erkennen"); ?> (1/4 &rarr; &#188;, &#8230;, 1 x 1 &rarr; 1&nbsp;&#215;&nbsp;1, &#8230;)</label></td>
					</tr>
					<tr valign="baseline">
						<td><input type="checkbox" id="ti_specialchar2" name="ti_specialchar2" value="1" <?php if ( $opt_specialchar2 == 1 ) echo "checked=\"checked\" " ?>/></td>
						<td><label for="ti_specialchar2"><?php ti_e("Sonderzeichen aus speziellen Zeichenfolgen erkennen - Teil 1"); ?> (-&gt; &rarr; &#8594;, &gt;= &rarr; &#8805;, o/oo &rarr; &#8240;, &#8230;)</label></td>
					</tr>
					<tr valign="baseline">
						<td><input type="checkbox" id="ti_specialchar3" name="ti_specialchar3" value="1" <?php if ( $opt_specialchar3 == 1 ) echo "checked=\"checked\" " ?>/></td>
						<td><label for="ti_specialchar3"><?php ti_e("Sonderzeichen aus speziellen Zeichenfolgen erkennen - Teil 2"); ?> (:-) &rarr; &#9786;, &#8230;)</label></td>
					</tr>
					<tr valign="baseline">
						<td><input type="checkbox" id="ti_speed" name="ti_speed" value="1" <?php if ( $opt_speed == 1 ) echo "checked=\"checked\" " ?>/></td>
						<td><label for="ti_speed"><?php ti_e("Schneller, aber ungenauer arbeiten"); ?></label></td>
					</tr>
				</tbody>
				</table>
			</fieldset>
			<fieldset class="options">
				<legend><?php ti_e("Alternative Anf&uuml;hrungszeichen"); ?></legend>
				<table cellspacing="2" cellpadding="5" class="editform">
				<tbody>
					<tr valign="baseline">
						<td><label for="ti_primquoteopen"><?php ti_e("Prim&auml;re Anf&uuml;hrungszeichen"); ?>:</label></td>
						<td><input type="text" id="ti_primquoteopen" name="ti_primquoteopen" value="<?php echo htmlentities(stripslashes($opt_primquoteopen), ENT_QUOTES, get_settings('blog_charset')) ?>" size="15" />
							<input type="text" id="ti_primquoteclose" name="ti_primquoteclose" value="<?php echo htmlentities(stripslashes($opt_primquoteclose), ENT_QUOTES, get_settings('blog_charset')) ?>" size="15" /><br />
							<?php ti_e("Prim&auml;r genutzte &ouml;ffnende und schlie&szlig;ende Anf&uuml;hrungszeichen"); ?></td>
					</tr>
					<tr valign="baseline">
						<td><label for="ti_secquoteopen"><?php ti_e("Sekund&auml;re Anf&uuml;hrungszeichen"); ?>:</label></td>
						<td><input type="text" id="ti_secquoteopen" name="ti_secquoteopen" value="<?php echo htmlentities(stripslashes($opt_secquoteopen), ENT_QUOTES, get_settings('blog_charset')) ?>" size="15" />
							<input type="text" id="ti_secuoteclose" name="ti_secquoteclose" value="<?php echo htmlentities(stripslashes($opt_secquoteclose), ENT_QUOTES, get_settings('blog_charset')) ?>" size="15" /><br />
							<?php ti_e("Sekund&auml;r genutzte &ouml;ffnende und schlie&szlig;ende Anf&uuml;hrungszeichen"); ?></td>
					</tr>
				</tbody>
				</table>
			</fieldset>
			<p class="submit"><input type="submit" name="ti_doit" value="<?php ti_e('Aktualisiere Optionen'); ?> &#187;" /></p>
		</form>

	</div>
<?php
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// adds the options page
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function ti_adminpage() {
	// add a new menu under options
	if ( function_exists('add_options_page') )
		add_options_page('Typographical Improvements', 'Typogr.-Impr.', 6, __FILE__, 'ti_options');
}



// ============================================================
// Integrate plugin
// ============================================================

add_action('init', 'ti_init');

// priority 9, just to guarantee that the filter starts before wptexturize()
add_filter('the_content', 'ti_replace', 9);
add_filter('the_excerpt', 'ti_replace', 9);
add_filter('the_title', 'ti_replace', 9);
add_filter('comment_text', 'ti_replace', 9);
add_action('admin_menu', 'ti_adminpage');

?>
